<?php
//
// Nagios XI Operations Center Component
// Copyright (c) 2011-2018 Nagios Enterprises, LLC. All rights reserved.
//

require_once(dirname(__FILE__) . '/../../common.inc.php');

// Initialization and session start
pre_init();
init_session();
grab_request_vars();
check_prereqs();

// Check authentication
check_authentication(false);

$host = grab_request_var("host", "");
$service = grab_request_var("service", "");
$hostgroup = grab_request_var("hostgroup", "");
$servicegroup = grab_request_var("servicegroup", "");
$state = grab_request_var("state", "");

$hide_soft = grab_request_var("hide_soft", "");
if ($hide_soft !== "") {
    set_user_meta(0, 'nocscreen_hide_soft_states', $hide_soft, false);
} else {
    $hide_soft = get_user_meta(0, 'nocscreen_hide_soft_states');
}

$hide_not_dis = grab_request_var("hide_not_dis", "");
if ($hide_not_dis !== "") {
    set_user_meta(0, 'nocscreen_hide_not_disabled', $hide_not_dis, false);
} else {
    $hide_not_dis = get_user_meta(0, 'nocscreen_hide_not_disabled');
}

$title = _("Operations Center");
do_page_start(array("page_title" => $title), true);
?>

<style type="text/css">
#topbar { margin-top: 10px; vertical-align: top; }
h1 { margin: 0; padding: 0; }
h1 img { margin-right: 10px; }
#leftside { float: left; }
#rightside { float: right; }
.strong { font-weight: bold; }
#lastUpdate { color: #777; font-size: 11px; margin: 10px 0; }
div.duration { width: 125px; }
#noc_filter{ margin: 0; }
.noc-filters { margin: 5px 0 15px 0; }
.filters-btn { margin-left: 10px; }
</style>

<script type="text/javascript">
REFRESH_RATE = 30; // Number of seconds between NOC refreshes

$(document).ready(function () {

    // Main NOC content and loop
    noc_load_content();

    // Hide/show filters
    $('.filters-btn').click(function() {
        if ($('.noc-filters').is(':visible')) {
            $('.noc-filters').hide();
            $('.filters-btn').html('<i class="fa fa-filter"></i> <?php echo _("Show Filters"); ?>');
        } else {
            $('.noc-filters').show();
            $('.filters-btn').html('<i class="fa fa-filter"></i> <?php echo _("Hide Filters"); ?>');
        }
    });

    // Add searchable drop-downs
    $('#hostList').searchable({maxMultiMatch: 9999});
    $('#hostgroupList').searchable({maxMultiMatch: 9999});
    $('#servicegroupList').searchable({maxMultiMatch: 9999});

    $('#hostList').change(function () {
        $('#hostgroupList').val('');
        $('#servicegroupList').val('');
    });

    $('#servicegroupList').change(function () {
        $('#hostList').val('');
        $('#hostgroupList').val('');
    });

    $('#hostgroupList').change(function () {
        $('#servicegroupList').val('');
        $('#hostList').val('');
    });

});

function noc_load_content()
{
    var datastring = $("#noc_filter").serialize();

    // Main NOC content
    $.ajax({
        url: 'nocscreenapi.php',
        data: datastring
    }).done(function (html) {
        var d = new Date();
        $('#lastUpdate').empty();
        $('#lastUpdate').append(('Last Update: ' + d.toString() ));
        $('#content').html(html);
    });

    // Summary bar info
    $.ajax({
        url: 'nocscreenapi.php?summary=true',
        data: datastring
    }).done(function (html) {
        $('#rightside').html(html);
    });

    setTimeout(noc_load_content, (REFRESH_RATE * 1000));
}
</script>

<div id="topbar">
    <div id="leftside">
        <h1>
            <img src="/nagiosxi/images/nagiosxi-logo-small.png" alt="<?php echo _("Nagios XI"); ?>">
            <?php echo _("Operations Center"); ?>
            <button type="button" class="filters-btn btn btn-xs btn-default"><i class="fa fa-filter"></i> <?php echo _("Hide Filters"); ?></button>
        </h1>
        <div id="lastUpdate"></div>
    </div>
    <div id="rightside"></div>
    <div class="clear"></div>
</div>

<div class="noc-filters">    
    <form id="noc_filter" method="get" action="<?php echo htmlentities($_SERVER["REQUEST_URI"]); ?>">
 
        <!-- options go here... -->
        <?php echo _("Limit To"); ?>&nbsp;

        <select name="host" id="hostList" class="form-control" style="width: 150px;">
            <option value=""><?php echo _("Host"); ?>:</option>
            <?php
            $args = array('brevity' => 1, 'orderby' => 'host_name:a');
            $oxml = get_xml_host_objects($args);
            if ($oxml) {
                foreach ($oxml->host as $hostobject) {
                    $name = strval($hostobject->host_name);
                    echo "<option value='" . $name . "' " . is_selected($host, $name) . ">$name</option>\n";
                }
            }
            ?>
        </select>

        <select name="hostgroup" id="hostgroupList" class="form-control" style="width: 150px;">
            <option value=""><?php echo _("Hostgroup"); ?>:</option>
            <?php
            $args = array('orderby' => 'hostgroup_name:a');
            $oxml = get_xml_hostgroup_objects($args);
            if ($oxml) {
                foreach ($oxml->hostgroup as $hg) {
                    $name = strval($hg->hostgroup_name);
                    echo "<option value='" . $name . "' " . is_selected($hostgroup, $name) . ">$name</option>\n";
                }
            }
            ?>
        </select>

        <select name="servicegroup" id="servicegroupList" class="form-control" style="width: 150px;">
            <option value=""><?php echo _("Servicegroup"); ?>:</option>
            <?php
            $args = array('orderby' => 'servicegroup_name:a');
            $oxml = get_xml_servicegroup_objects($args);
            if ($oxml) {
                foreach ($oxml->servicegroup as $sg) {
                    $name = strval($sg->servicegroup_name);
                    echo "<option value='" . $name . "' " . is_selected($servicegroup, $name) . ">$name</option>\n";
                }
            }
            ?>
        </select>

        <select name="state" id="state" class="form-control">
            <option value=""><?php echo _("Service State"); ?>:</option>
            <?php
                    echo "<option value='1' " . is_selected($state, 1) . ">"._("Warning")."</option>\n";
                    echo "<option value='2' " . is_selected($state, 2) . ">"._("Critical")."</option>\n";
                    echo "<option value='3' " . is_selected($state, 3) . ">"._("Unknown")."</option>\n";
            ?>
        </select>

        <select name="hide_soft" id="hide_soft" class="form-control">
            <option value="0"><?php echo _("State"); ?>:</option>
            <?php
                    echo "<option value='0' " . is_selected($hide_soft, 0) . ">"._("Show All States")."</option>
                          <option value='1' " . is_selected($hide_soft, 1) . ">"._("Hide Soft States")."</option>\n";
            ?>
        </select>

        <select name="hide_not_dis" id="hide_not_dis" class="form-control">
            <option value="0" <?php echo is_selected($hide_not_dis, 0); ?>><?php echo _("Extra Filters"); ?>:</option>
            <?php
                    echo "<option value='1' " . is_selected($hide_not_dis, 1) . ">"._("Hide notifications disabled")."</option>";
            ?>
        </select>

        <button type="submit" class="reporttimesubmitbutton btn btn-sm btn-primary" style="vertical-align: top;" name="reporttimesubmitbutton"><?php echo _("Update"); ?></button>

    </form>
    <div class="clear"></div>
</div>

<div id="content"></div>

<?php
do_page_end(true);
